/*==========================================================================
//  Implementation of Generalized Benchmark Generator for Dynamic Problems
//
//  See the details of GDBG in "Generalized Dynamic Benchmark Generator for CEC'2009 
//  Competition on Dynamic Optimization" 
//
//  The source codes of GDBG was implemented by Changhe Li 
//
//  If you have any questions about the codes, please contact 
//  Changhe Li at cl160@le.ac.uk
===========================================================================*/


#include "Global.h"
#include "Composition_DBG.h"
#include "Rotation_DBG.h"


#include <fstream>
#include <iostream>
using namespace std;

void Print_Change_Infor(){
	cout<<"please input change type:"<<endl;
	cout<<"		0: small step"<<endl;
	cout<<"		1: large step"<<endl;
	cout<<"		2: random"<<endl;
	cout<<"		3: recurrent"<<endl;
	cout<<"		4: chaotic"<<endl;
	cout<<"		5: recurrent random"<<endl;
	
}
void Print_System_Infor(bool f){
	if(f==false)
	cout<<"***Welcome to Generalized Dynamic Benchmark Generator***"<<endl;
	cout<<" Please press any key of following:"<<endl;
	cout<<"     1 : Dynamic Composition Benchmark Generator for continuous problems "<<endl;
	cout<<"     2 : Dynamic Rotation Benchmark Generator for continuous problems"<<endl;
	cout<<"     Q : Exit the system "<<endl;
}

void Initial_Change_Counter(){
	for(int i=0;i<Num_Change_Type;i++){
		Global::change_type[i].type=(Change_type) i;
		Global::change_type[i].counter=0;
	}
}
void System_Initial(double seed){
	
	static LGM_mixed urng(seed);
	Random::Set(urng);
	Global::num_change=100;

	Global::min_dimension=5;
	Global::max_dimension=15;
	Global::num_dim=10;
	while(Global::num_dim<5||Global::num_dim>15){
		cout<<"please input dimensions within [5,15]:";
		cin>>Global::num_dim;
		getchar();
	}
	Global::boundary.Set_Boundary(-5,5);
	Global::change_frequency=10000;
	Print_Change_Infor();
	char c;
	int t;
	cin>>c;
	t=c-48;
	getchar();
	while(t<0||t>5){
		Print_Change_Infor();
		cin>>c;
		t=c-48;
		getchar();
	}
	Global::change=(Change_type)(t);
	if(t==recurrent||t==recurrent_noisy)
		Global::periodicity=12;
	else
		Global::periodicity=0;

	Initial_Change_Counter();
}
void Continuous_Setting(Real_DBG *p){
	p->Set_Boundary(Global::boundary);
	double *t=new double[Global::num_peakorfun];
	
	for(int i=0;i<Global::num_peakorfun;i++){
		if(p->Get_Change_Type()==chaotic)
			t[i]=Global::min_height+(Global::max_height-Global::min_height)*Global::uniform.Next();
		else
			t[i]=50;
	}
	p->Set_Height(t);
	p->Set_Height_Severity(5.0);

	double **position;
		position=new double*[Global::num_peakorfun];
		for(int i=0;i<Global::num_peakorfun;i++)
			position[i]=new double[Global::num_dim];
		for(int i=0;i<Global::num_peakorfun;i++)
			for(int j=0;j<Global::num_dim;j++){
				position[i][j]=p->Get_Boundary()[j].lower+(p->Get_Boundary()[j].upper-p->Get_Boundary()[j].lower)*Global::uniform.Next();	
			}
		p->Set_Position(const_cast<const double **>(position));
	delete [] t;
	for(int i=0;i<Global::num_peakorfun;i++)
			delete []position[i];
	delete [] position;
}
void Rotation_DBG_Setting(Rotation_DBG *p){

	Continuous_Setting(p);
	p->Set_Width_Severity(0.5);
	p->Set_Weight(5);		// between (1,10)
	p->Calculate_Global_Optima();
}
void Composition_DBG_Setting(Composition_DBG * p){
		
	Continuous_Setting(p);

	Fun_Name *basic_fun=new Fun_Name[Global::num_peakorfun];
	for(int i=0;i<Global::num_peakorfun;i++) basic_fun[i]=Sphere;
	p->Set_Basic_Function(basic_fun);
	double *t=new double[Global::num_peakorfun];
	for(int i=0;i<Global::num_peakorfun;i++)t[i]=1.;
	p->Set_Coverge_Sevrity(t);
	p->Set_Stretch_Severity();		
	p->Set_Rotation_Matrix();
	p->Calculate_Global_Optima();
	delete []basic_fun;
	delete [] t;
		

}

void System_Setting(General_DBG * g_dbg){

	g_dbg->Set_Change_Frequency(Global::change_frequency);
	g_dbg->Set_Change_Type(Global::change);

	g_dbg->Set_Periodicity(Global::periodicity);
	
	char c;
	cout<<"The number of dimensions changes or not ?(Y/N): ";
	cin>>c;
	getchar();
	while(c!='N'&&c!='n'&&c!='y'&&c!='Y'){
		cout<<"please input Y/N: ";
		cin>>c;
		getchar();
	}
	if(c=='Y'||c=='y')
		g_dbg->Set_Dimension_Change(true);
	else
		g_dbg->Set_Dimension_Change(false);

	if(g_dbg->Get_Change_Type()==chaotic) {
		Global::chaotic_constant=3.67;
		while(Global::chaotic_constant>4||Global::chaotic_constant<1){
			cout<<"invalid value of chaotic_constant,reset please"<<endl;
			cin>>Global::chaotic_constant;
			getchar();
		}
	}

	if(g_dbg->Get_Change_Type()==recurrent_noisy)
		g_dbg->Set_RR_Severity(0.8f);

	if(Composition_DBG * p=dynamic_cast<Composition_DBG*>(g_dbg)){
		Composition_DBG_Setting(p);
	}else if(Rotation_DBG * p=dynamic_cast<Rotation_DBG*>(g_dbg)){
		Rotation_DBG_Setting(p);
	}

}

void Test_Composition_DBG(){
	Composition_DBG * com_dbg;
	Global::num_peakorfun=10;
	Global::hn_s=2000.;
	Global::min_height=10;
	Global::max_height=100;

	com_dbg=Composition_DBG::Get_Composition_DBG();
	System_Setting(com_dbg);

	ofstream fun("fun.txt");
	for(int g=1;g<=com_dbg->Get_Change_Frequency()*Global::num_change;g++)
		if(g%com_dbg->Get_Change_Frequency()==0){		
			// generate a solution 
			double *x=new double[Global::num_dim];
			for(int i=0;i<Global::num_dim;i++)
			x[i]=com_dbg->Get_Boundary()[i].lower+(com_dbg->Get_Boundary()[i].upper-com_dbg->Get_Boundary()[i].lower)*Global::uniform.Next();
			double fit=com_dbg->Evaluation(x);
			cout<<fit<<endl;
			
			com_dbg->Change();
			
			delete []x;
			fun<<"the change of number "<<com_dbg->Get_Changes_Counter()<<" :"<<endl;
			com_dbg->Print_Fun(fun);
			if(com_dbg->Get_Dimension_Change_Flag()==true) {
				com_dbg->Dimension_Change();
				com_dbg->Delete_Composition_DBG(); 
				com_dbg=dynamic_cast<Composition_DBG*>(Global::g_dbg);
			}
		}
	fun.close();
	com_dbg->Delete_Composition_DBG();
	
}
void Test_Rotation_DBG(){
	Rotation_DBG *rot_dbg;
	Global::num_peakorfun=10;
	Global::max_width=10.;
	Global::min_width=1;
	Global::min_height=10;
	Global::max_height=100;

	rot_dbg= Rotation_DBG::Get_Rotation_DBG();
	System_Setting(rot_dbg);

	ofstream fun("fun.txt");

	for(int g=1;g<=rot_dbg->Get_Change_Frequency()*Global::num_change;g++)
		if(g%rot_dbg->Get_Change_Frequency()==0){
			
			//generate a solution
			double *x=new double[Global::num_dim];
			for(int i=0;i<Global::num_dim;i++)
			x[i]=rot_dbg->Get_Boundary()[i].lower+(rot_dbg->Get_Boundary()[i].upper-rot_dbg->Get_Boundary()[i].lower)*Global::uniform.Next();
			double fit=rot_dbg->Evaluation(x);
			cout<<fit<<endl;
			delete []x;

			rot_dbg->Change();
			
			fun<<"the "<<rot_dbg->Get_Changes_Counter()<<" times change:"<<endl;
			
			rot_dbg->Print_Fun(fun);
			
			if(rot_dbg->Get_Dimension_Change_Flag()==true){
				rot_dbg->Dimension_Change();
				rot_dbg->Delete_Rotation_DBG(); 
				rot_dbg=dynamic_cast<Rotation_DBG*>(Global::g_dbg);
			}
			
		}
	fun.close();

	rot_dbg->Delete_Rotation_DBG();
}
int main(){
	bool flag=false;
	char c;
	while(1){	
		Print_System_Infor(flag);
		cin>>c;
		getchar();
		switch(c){
			case '1':
				System_Initial(0.485);
				Test_Composition_DBG();
				break;
			case '2':
				System_Initial(0.485);
				Test_Rotation_DBG();
				break;
			case 'Q':
			case 'q':
				break;
			default:
				flag=true;
				break;
		}
	
	if(c=='Q'||c=='q') break;
	
	}
return 0;
	
}
