#include "Swarm.h"

#include <iostream>
#include "Global.h"

#include "Composition_DBG.h"
#include "Rotation_DBG.h"

using namespace std;

CParticle *CSwarm::global_best;

int CSwarm::pop_num=0;
CSwarm *CSwarm::sub_swarm=0;
int CSwarm::max_popsize=50;
const int CSwarm::max_popnum=1;


CSwarm::CSwarm(void){
	
}
CSwarm::CSwarm(const CSwarm &s){
	int i;
	popsize=s.popsize ;
	pop= new CParticle[popsize];
	pop_best = new CParticle[popsize];

	for( i=0;i<popsize;i++){
		pop[i]=s.pop[i];
		pop_best[i]=s.pop_best[i];
	}
	
	Best=s.Best;
	
	avg_fitness=s.avg_fitness;
	std_error=s.std_error;

}
CSwarm::CSwarm(const int pop_size){
	popsize=pop_size;
	pop= new CParticle[pop_size];
	pop_best = new CParticle[pop_size];
}
CSwarm &CSwarm::operator= (const CSwarm &s){
	if(this==&s)return *this;

	int i;
	popsize=s.popsize ;
	for( i=0;i<popsize;i++){
		pop[i]=s.pop[i];
		pop_best[i]=s.pop_best[i];
	}
	Best=s.Best;	
	avg_fitness=s.avg_fitness;
	std_error=s.std_error;
	return *this;
}
CSwarm::~CSwarm(void){
	if(pop){
		delete [] pop;
		pop=0;
	}
	if(pop_best) {
		delete [] pop_best;
		pop_best=0;
	}
}

void CSwarm::Initial(const int pop_size){
	int i;

	popsize=pop_size;

	pop= new CParticle[pop_size];
	for( i=0;i<pop_size;i++) pop[i].Initialization();
		
	pop_best = new CParticle[pop_size];
	for( i=0;i<pop_size;i++)
		pop_best[i]=pop[i];
	
	Best=pop[Find_Best()];

}
void CSwarm::Reinital(){
	int i;
	for( i=0;i<popsize;i++) pop[i].Initialization();
	for( i=0;i<popsize;i++)
		pop_best[i]=pop[i];
	Best=pop[Find_Best()];

}
const int CSwarm::Find_Best(void)const
{
	int index_best=0;
	for(int i=1;i<popsize;i++){
		if(pop[i].Comparison(pop[index_best])==1){
			index_best=i;
		}
	}
	return index_best;
}
void CSwarm::Move(double best, int &fit_eva, double &r_value){
	for(int i=0;i<popsize;i++){
		pop[i].PSO_Move(pop_best[i],Best);
		fit_eva++;
		if(fit_eva%Global::sample_frequency==0){
			if(Global::optimization_type==MIN)
				r_value+=(1-best/CSwarm::global_best->fitness);
			else
				r_value+=(1-CSwarm::global_best->fitness/best);
		}
		if(pop[i].Comparison(pop_best[i])==1) {
			pop_best[i]=pop[i];
		}
	}

}

void CSwarm::Statistic(void)
{
	Calculate_Avgfit();
	std_error=0;
	for(int i=0;i<popsize;i++)
		std_error+=(pop[i].fitness-avg_fitness)*(pop[i].fitness-avg_fitness);
	std_error=sqrt(std_error/popsize);
}

void CSwarm::Calculate_Avgfit(void){
	avg_fitness=0;
	// calculate the average fitness of its swarm
	for(int i=0;i<popsize;i++)
		avg_fitness+=pop[i].fitness;
	avg_fitness/=popsize;
}

void CSwarm::Update_Best_Pbest(){
	
	int b=Find_Best();
	if(pop[b].Comparison(Best)==1)
	Best=pop[b];

	// update the best one of all swarms
	if(Best.Comparison(*global_best)==1)
		global_best=&Best;
}
void CSwarm::Add_Pop(CSwarm & w){
	CSwarm *sw=new CSwarm[pop_num+1];
	int i=0;
	for( i=0;i<pop_num;i++){
		sw[i].Initial(sub_swarm[i].popsize);
		sw[i]=sub_swarm[i];
	}
	sw[i].Initial(w.popsize);
	sw[i]=w;
	delete [] sub_swarm;
	pop_num++;
	sub_swarm= sw;

}
void CSwarm::Delete_Pop(int index){
	if(pop_num==1) {
		delete  [] sub_swarm;
		sub_swarm=0;
		pop_num--;
		return ;
	}
	CSwarm *sw=new CSwarm[pop_num-1];
	int i,j;
	for( i=0,j=0;j<pop_num;i++,j++){
		if(j==index){
			i--;
			continue;	
		}
		sw[i].Initial(sub_swarm[j].popsize);
		sw[i]=sub_swarm[j];
	}
	delete [] sub_swarm;
	pop_num--;
	sub_swarm=sw;
}
void CSwarm::Add_Particle(CParticle &add){
	CParticle *sw=new CParticle[popsize+1];
	CParticle *swbest=new CParticle[popsize+1];
	int i;
	for( i=0;i<popsize;i++){
		sw[i]=pop[i];
		swbest[i]=pop_best[i];
	}
	sw[i]=add;
	swbest[i]=add;
	
	delete [] pop;
	pop=0;
	delete [] pop_best;
	pop_best=0;
	pop=sw;
	pop_best=swbest;
	popsize++;
}
void CSwarm::Delete_Particle(int del){
	if(popsize==1) {
		delete  [] pop;
		delete []pop_best;
		pop_best=0;
		pop=0;
		popsize--;
		return;
	}
	CParticle *sw=new CParticle[popsize-1];
	CParticle *swbest=new CParticle[popsize-1];
	int i,j;
	for( i=0,j=0;j<popsize;i++,j++){
		if(j==del){
			i--;
			continue;	
		}
		sw[i]=pop[j];
		swbest[i]=pop_best[j];
	}
	delete [] pop;
	pop=0;
	delete []pop_best;
	pop_best=0;
	pop=sw;
	pop_best=swbest;
	popsize--;
}

